/**
 * Copyright (C) Altimetrik 2018. All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Altimetrik. You shall not disclose such Confidential Information
 * and shall use it only in accordance with the terms and conditions
 * entered into with Altimetrik.
 */

package com.altimetrik.playground.innovation.security;

import java.io.IOException;
import java.util.List;
import java.util.concurrent.TimeUnit;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.authentication.AnonymousAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.AuthorityUtils;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.web.authentication.AnonymousAuthenticationFilter;

import com.altimetrik.playground.innovation.bean.Principal;
import com.google.common.base.Stopwatch;

import lombok.extern.slf4j.Slf4j;

/**
 * Detects if there is no {@code Authentication} object in the
 * {@code SecurityContextHolder}, and populates it with gateway authenticated
 * user details.
 *
 * @author Altimetrik Technologies
 */
@Slf4j
public class GatewayAuthenticationFilter extends AnonymousAuthenticationFilter {

	private static final String SPACE = " ";

	/** The default user. */
	@Value("${regulus.security.defaultUser:#{'anonymoususer'}}")
	private String defaultUser;

	/** The default realm. */
	@Value("${regulus.security.defaultRealm:#{'anonymous'}}")
	private String defaultRealm;

	/** The default roles. */
	@Value("${regulus.security.defaultRoles:#{'ROLE_anonymous'}}")
	private String defaultRoles;

	/**
	 * Instantiates a new gateway authentication filter.
	 *
	 * @param key the key
	 */
	public GatewayAuthenticationFilter(String key) {
		super(key);
	}

	@Override
	public void doFilter(ServletRequest req, ServletResponse res, FilterChain chain) throws IOException, ServletException {
		HttpServletRequest request = (HttpServletRequest) req;
		String principal = StringUtils.defaultIfBlank(request.getHeader("x-regulus-user"), this.defaultUser);
		if (SecurityContextHolder.getContext().getAuthentication() == null) {
			SecurityContextHolder.getContext().setAuthentication(createAuthentication(request));
			if (log.isDebugEnabled()) {
				log.debug("Populated SecurityContextHolder with gateway authenticated user: '" + SecurityContextHolder.getContext().getAuthentication() + "'");
			}
		}
		else {
			if (log.isDebugEnabled()) {
				log.debug("SecurityContextHolder not populated with gateway authenticated user, as it already contained: '" + SecurityContextHolder.getContext().getAuthentication()
						+ "'");
			}
		}
		if (SecurityContextHolder.getContext().getAuthentication().getPrincipal() instanceof Principal) {
			Principal principalObj = (Principal) SecurityContextHolder.getContext().getAuthentication().getPrincipal();
			principal = principalObj.getUserName();
		}
		Stopwatch stopwatch = Stopwatch.createStarted();
		chain.doFilter(req, res);
		stopwatch.stop();
		createTraceEvent(principal, stopwatch.elapsed(TimeUnit.MILLISECONDS), request);
	}

	/**
	 * Creates the authentication.
	 * @param request the request
	 * @return the authentication
	 */
	@Override
	protected Authentication createAuthentication(HttpServletRequest request) {
		String principal = StringUtils.defaultIfBlank(request.getHeader("x-regulus-user"), this.defaultUser);
		String key = StringUtils.defaultIfBlank(request.getHeader("x-regulus-user-realm"), this.defaultRealm);
		boolean notAuthenticated = "false".equalsIgnoreCase(request.getHeader("x-regulus-user-authenticated"));

		String authorityString = StringUtils.defaultIfBlank(request.getHeader("x-regulus-user-authorities"), this.defaultRoles);
		List<GrantedAuthority> authorities = AuthorityUtils.commaSeparatedStringToAuthorityList(authorityString);

		AnonymousAuthenticationToken authentication = new AnonymousAuthenticationToken(key, principal, authorities);
		authentication.setAuthenticated(!notAuthenticated);
		return authentication;
	}

	protected void createTraceEvent(String principal, long millis, HttpServletRequest request) {
		StringBuilder message = new StringBuilder();
		message.append(StringUtils.defaultIfBlank(request.getHeader("X-Real-IP"), "-") + SPACE);
		message.append(principal + SPACE);
		message.append(request.getRequestURI() + SPACE);
		message.append(request.getMethod() + SPACE);
		message.append(StringUtils.defaultIfBlank(request.getHeader("X-Request-ID"), "-") + SPACE);
		message.append(millis);
		log.info(message.toString());
	}

}
